package com.acme.client;

import com.acme.client.dnd.ListDragPanel;
import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.EventPreview;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.KeyboardListenerAdapter;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.UIObject;
import com.google.gwt.user.client.ui.Widget;

/**
 * The entry point for the Yahoo! Trips application.
 */
public class YahooTripsApplication implements EntryPoint {
  /**
   * This is the entry point method, sort of like a
   * main() method.
   */
  public void onModuleLoad() {
   // Create the textbox and button for our simple UI
    final TextBox textBox = new TextBox();
    final Button button = new Button("Search");

    // Initialize the textBox and add a keyboard listener
    // that clicks the button when the user types ENTER.
    textBox.setVisibleLength(20);
    textBox.setMaxLength(100);
    textBox.addKeyboardListener(new KeyboardListenerAdapter() {
        public void onKeyPress(Widget sender, 
                             char keyCode, int mods) {
            if (keyCode == KEY_ENTER)
                button.click();
        }
    });

    // We can't give the textBox focus now because it hasn't
    // been displayed. So we create a deferred command that
    // gives focus to the textBox after the current event
    // stack has been processed (and the textBox has been
    // displayed).
    DeferredCommand.add(new Command() {
        public void execute() {
            textBox.setFocus(true);
        }
    });
    
    // Add a click listener to the button that makes an RPC
    // to get trips from the Yahoo! Trips web service.
    button.addClickListener(new ClickListener() {
       public void onClick(Widget sender) {
   
         // Get the service from the GWT
         TripServiceAsync ws = (TripServiceAsync)
             GWT.create(TripService.class);
   
         // The service is also an instance of 
         // ServiceDefTarget. We take advantage of
         // that by setting the service's entry point.
         //
         // Note that the entry point -- /tripService --
         // must match the servlet mapping in either
         // your GWT config file (for hosted mode) or
         // your /WEB-INF/web.xml file (for web mode)
         ServiceDefTarget target = (ServiceDefTarget) ws;
         target.setServiceEntryPoint(GWT.getModuleBaseURL()
                                     + "/tripService");
         
         // Invoke the service. GWT calls either onSuccess()
         // or onFailure() after the RPC completes.
         // 
         // The result that GWT passes back from the server
         // is an array of com.acme.client.Trip objects, which
         // implement the GWT's IsSerializable interface.
         ws.getTrips(textBox.getText(), new AsyncCallback() {
             public void onSuccess(Object result) {
               // Get the trips and create a list drag panel
                Trip[] trips = (Trip[])result;
                final ListDragPanel p = new ListDragPanel();
                
                p.addStyleName("tripsPanel");
                p.setWidth("800px");
                p.setHeight("800px");
              
                for (int i=0; i < trips.length; ++i) {
                    TripPanel tp = new TripPanel(trips[i]);
                    tp.addStyleName("tripDetailsPanel");
                    tp.setWidth("800px");
                    p.add(tp);
                }
   
                final ScrollPanel sp = new ScrollPanel(p);
                sp.setWidth("800px");
                sp.setHeight("400px");
   
                final VerticalPanel vp = new VerticalPanel();
                Label heading = new Label("Trips to " 
                                        + textBox.getText());
                vp.add(heading);
                vp.add(sp);
                vp.addStyleName("tripPanel");
                
                RootPanel.get().add(vp);
   
                p.setVisible(false);
                applyEffect("Grow", p.getElement());
             }
             public void onFailure(Throwable t) {
               t.printStackTrace();
               showAlert("RPC call failed: " + t.getMessage()
                          + " stack trace: " + t.getStackTrace());
             }
         });
       }
    });
    
    // Add an event preview to the event stack that
    // cancel's the browser's default actions (such
    // as selecting images) when we drag the mouse.
    //
    // The event preview only cancels the browser's
    // default actions if the source of the event was
    // not the textbox or button in our user interface.
    //
    // The event preview's onEventPreview method always
    // returns true. You can return false from 
    // onEventPreview to cancel the event entirely.
    DOM.addEventPreview(new EventPreview() {
        public boolean onEventPreview(Event event) {
           if ( ! isEventTarget(event, button) &&
               ! isEventTarget(event, textBox)) {
                switch (DOM.eventGetType(event)) {
                    case Event.ONMOUSEDOWN:
                    case Event.ONMOUSEUP:
                    case Event.ONMOUSEMOVE:
                        DOM.eventPreventDefault(event);
                }
            }
            return true; 
        }
        /*
         * A simple convenience method to make the preceeding
         * method more readable.
         */
        private boolean isEventTarget(Event event, UIObject uio) {
            return DOM.eventGetTarget(event) == uio.getElement();
        }
    });

    // Create a horizontal panel that contains a prompt and
    // the textBox.
    final HorizontalPanel hp = new HorizontalPanel();
    hp.add(new HTML("Search for trips to&nbsp;"));
    hp.add(textBox);
    
    // Create a vertical panel that contains the horizontal
    // panel and the button.
    final VerticalPanel vp = new VerticalPanel();    
    vp.add(hp);
    vp.add(button);

    // Add the vertical panel to the root panel.
    RootPanel.get().add(vp);
  }
  private native void showAlert(String message) /*-{
      alert(message);
  }-*/;
  private native void applyEffect(String effect,
                                  Element element) /*-{
      var ne = $wnd._nativeExtensions;
      $wnd._nativeExtensions = false;
      $wnd.Effect[effect](element);
      $wnd._nativeExtensions = ne;
  }-*/;
}
