package com.acme.client.dnd;

import java.util.Iterator;
import java.util.Vector;

import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.MouseListener;
import com.google.gwt.user.client.ui.Widget;

public class ListDragPanel extends DragPanel {
	private final Vector rows = new Vector();
	
	public void onLoad() {
		Iterator it = iterator();
		int level = 0;
		while(it.hasNext()) {
			Widget w = (Widget)it.next();
			setWidgetPosition(w, 0, level);
			rows.add(new Row(w, level));
			level += w.getOffsetHeight();
		}
	}
	protected MouseListener getMouseDragger(Widget widget) {
		return new ListMouseDragger();
	}
	private class ListMouseDragger extends VerticalMouseDragger {
		public void onMouseDown(Widget widget, int x, int y) {
			super.onMouseDown(widget, x, y);
		}
		public void onMouseUp(Widget widget, int x, int y) {
			super.onMouseUp(widget, x, y);
			
			int index = getIndexFor(widget);
			Row row = (Row)rows.elementAt(index);
			AbsolutePanel panel = (AbsolutePanel)widget.getParent();
			panel.setWidgetPosition(widget, 0, row.top);
		}
		public void onMouseMove(Widget widget, int x, int y) {
			super.onMouseMove(widget, x, y);
			shuffle(widget);
		}
		private void shuffle(Widget widget) {
			AbsolutePanel panel = (AbsolutePanel)widget.getParent();
			int widgetIndex = getIndexFor(widget);
			int top = panel.getWidgetTop(widget); // top of moving widget
			int topIndex = getIndexAt(top);
			Row topRow = (Row)rows.elementAt(topIndex);
			Widget topWidget= topRow.widget;
			
			if (topIndex < widgetIndex) { // moving up
				int thresh = panel.getWidgetTop(topWidget) +
				             topWidget.getOffsetHeight()/2;
					
				for (int i = widgetIndex-1; i >= topIndex; --i) {
					if (i == topIndex && top > thresh)
						return;
						
					Row row = (Row)rows.elementAt(i);
					moveDown(row.widget);
				}
			}
			else { // moving down
				int bottom = top + topWidget.getOffsetHeight();
				int bottomIndex = getIndexAt(bottom);
				Row bottomRow = (Row)rows.elementAt(bottomIndex);
				Widget bottomWidget = bottomRow.widget;
				int threshold = panel.getWidgetTop(bottomWidget) 
				             + bottomWidget.getOffsetHeight()/2;
				
				for (int i = widgetIndex+1; i <= bottomIndex; ++i) {
					if (i == bottomIndex && bottom < threshold) {
						return;
					}
					Row row = (Row)rows.elementAt(i);
					moveUp(row.widget);
				}
			}
		}
		private void moveDown(Widget widget) {
			AbsolutePanel panel = (AbsolutePanel)widget.getParent();
			int widgetIndex = getIndexFor(widget);
			
			if (widgetIndex < rows.size()-1) {
				Row widgetRow = (Row)rows.remove(widgetIndex);
				Row rowBelow = (Row)rows.elementAt(widgetIndex);
				
				rows.insertElementAt(widgetRow, widgetIndex+1);
				widgetRow.top = rowBelow.top + rowBelow.widget.getOffsetHeight()
				                             - widgetRow.widget.getOffsetHeight();
				rowBelow.top = widgetRow.top - 
								rowBelow.widget.getOffsetHeight();
				
				panel.setWidgetPosition(widget, 0, widgetRow.top);
			}
		}
		private void moveUp(Widget widget) {
			AbsolutePanel panel = (AbsolutePanel)widget.getParent();
			int widgetIndex = getIndexFor(widget);
			
			if (widgetIndex != 0) {
				Row widgetRow = (Row)rows.remove(widgetIndex);
				Row rowAbove = (Row)rows.elementAt(widgetIndex-1);
				rows.insertElementAt(widgetRow, widgetIndex-1);
				
				widgetRow.top = rowAbove.top;
				rowAbove.top = widgetRow.top + widget.getOffsetHeight();
				
				panel.setWidgetPosition(widget, 0, widgetRow.top);
			}
		}
		private int getIndexFor(Widget widget) {
			Iterator it = ListDragPanel.this.rows.iterator();
			
			for (int index = 0; it.hasNext(); index++) {
				Row row = (Row)it.next();
				if (row.widget == widget)
					return index;
			}
			return 0;
		}
		private int getIndexAt(int height) {
			Iterator it = ListDragPanel.this.rows.iterator();
			
			for (int index = 0; it.hasNext(); index++) {
				Row row = (Row)it.next();
				Widget w = row.widget;
				int h = w.getOffsetHeight();
				int top = row.top;
				
				if (height >= top && height <= top+h)
					return index;
			}
			return 0;
		}
	}
	private class Row {
		public Widget widget;
		public int top;
		
		public Row(Widget widget, int top) {
			this.widget = widget;
			this.top = top;
		}
	}
}
