package com.acme.client;

import java.util.Iterator;
import java.util.List;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.DeferredCommand;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Hyperlink;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * The address book application.
 */
public class AddressBookApplication implements EntryPoint {
    private final AddressBookServiceAsync ws = 
       (AddressBookServiceAsync)
           GWT.create(AddressBookService.class);
    private final ServiceDefTarget target = 
           (ServiceDefTarget) ws;
    private final Grid grid = new Grid();
    
  /**
   * This is the entry point method.
   */
  public void onModuleLoad() {
     final Label loadingLabel = (new Label("loading contacts "
                                + "from the database..."));
     final Label headingLabel = new Label("Your contacts:");
     
     target.setServiceEntryPoint(GWT.getModuleBaseURL() 
            + "/addressBookService");
     RootPanel.get().add(loadingLabel);
     grid.addStyleName("contactsTable");
     DeferredCommand.add(new Command() {
       public void execute() {
          loadAddressBook();
          HorizontalPanel hp = new HorizontalPanel();
          hp.add(headingLabel);
          
          VerticalPanel vp = new VerticalPanel();
          Button newContactButton = 
             new Button("New contact...");
         
          newContactButton.addClickListener(new ClickListener() {
            public void onClick(Widget sender) {
               createContact(sender);
            }             
          });
          vp.add(grid);
          vp.add(newContactButton);
          
          hp.add(vp);
          
          RootPanel.get().remove(loadingLabel);
          RootPanel.get().add(hp);
       }
     });
  }
  private void createContact(Widget sender) {
     PopupPanel popup = new PopupPanel(true);
     popup.addStyleName("createContactPopup");
     popup.setPopupPosition(sender.getAbsoluteLeft() + 10,
                          sender.getAbsoluteTop() + 10);
     Panel p = createContactPanel(popup, null, 
                                  "Create a contact");
     popup.add(p);
     popup.show();
  }
  private Panel createContactPanel(final PopupPanel panel,
                             final Contact contact,
                             final String headingText) {
     final VerticalPanel vp = new VerticalPanel();
     final Grid grid = new Grid(5, 2);
     final Label heading = new Label(headingText);
     final Label namePrompt = new Label("Name: ");
     final TextBox nameTextBox = new TextBox();
     final TextBox emailTextBox = new TextBox();
     final Label emailPrompt = new Label("Email: ");
     final Button button = new Button(headingText);
     
     heading.addStyleName("heading");
     
     DeferredCommand.add(new Command() {
        public void execute() {
           nameTextBox.setFocus(true);
        }
     });
     
     if (contact != null) {
        nameTextBox.setText(contact.getName());
        emailTextBox.setText(contact.getEmail());
     }
     button.addClickListener(new ClickListener() {
      public void onClick(Widget sender) {
         String name = nameTextBox.getText();
         String email = emailTextBox.getText();
         
         if (contact != null) {
            contact.setName(name);
            contact.setEmail(email);
            updateContact(contact);
         }
         else
            saveContact(new Contact(null, name, email));
         
         panel.hide();
      }
     });
     grid.setWidget(0, 0, namePrompt);     
     grid.setWidget(0, 1, nameTextBox);     
     grid.setWidget(1, 0, emailPrompt);     
     grid.setWidget(1, 1, emailTextBox);
     
     vp.add(heading);
     vp.add(grid);
     vp.add(button);
     return vp;
  }
  private void deleteContact(Contact contact) {
   ws.deleteContact(contact, new AsyncCallback() {
      public void onSuccess(Object result) {
         loadAddressBook();
      }
      public void onFailure(Throwable t) {   
         showAlert("Delete failed");
      }
   });     
  }
  private void editContact(Widget sender, Contact contact) {
     PopupPanel popup = new PopupPanel(true);
     popup.addStyleName("createContactPopup");
     popup.setPopupPosition(sender.getAbsoluteLeft() + 10,
                            sender.getAbsoluteTop() + 10);
     Panel p = createContactPanel(popup, contact,
                                  "Edit Contact");
     popup.add(p);
     popup.show();     
  }
  private void updateContact(Contact contact) {
   ws.updateContact(contact, new AsyncCallback() {
      public void onSuccess(Object result) {
         loadAddressBook();
      }
      public void onFailure(Throwable t) {         
      }
   });
  }
  private void saveContact(Contact contact) {
   ws.saveContact(contact, new AsyncCallback() {
      public void onSuccess(Object result) {
         loadAddressBook();
      }
      public void onFailure(Throwable t) {         
      }
   });
  }
  private void loadAddressBook() {
   ws.getContacts(new AsyncCallback() {
      public void onSuccess(Object result) {
         List contacts = (List)result;
         Iterator it = contacts.iterator();

         grid.clear();
         
         if (contacts.size() == 0) {
            grid.resize(1, 1);
            grid.setText(0, 0, "You don't have any contacts");
            return;
         }
         
         grid.resize(contacts.size(), 3);
         
         for (int row=0; it.hasNext(); ++row) {
            final Contact c = (Contact)it.next();
            Button b = new Button("delete");
            Hyperlink h = new Hyperlink(c.getName(), 
                                        c.getName());

            grid.setWidget(row, 0, h); // edit hyperlink
            grid.setText  (row, 1, c.getEmail());  // email            
            grid.setWidget(row, 2, b); // delete button
            
            h.addClickListener(new ClickListener() {
               public void onClick(Widget sender) {
                  editContact(sender, c);
               }
            });
            
            b.addClickListener(new ClickListener() {
               public void onClick(Widget sender) {
                  deleteContact(c);
               }
            });
         }
      }
      public void onFailure(Throwable t) {
         t.printStackTrace();
         showAlert("darnit " + t.getMessage());
      }
   });
  }
  private native void showAlert(String s) /*-{
   $wnd.alert(s);
  }-*/;
}
