package com.acme.client;

import com.acme.client.dnd.FollowsMouseDragger;
import com.acme.client.dnd.ListDragPanel;
import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.user.client.*;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.*;

/**
 * The Yahoo Trips application.
 */
public class YahooTripsApplication implements EntryPoint {

  /**
   * This is the entry point method.
   */
  public void onModuleLoad() {
    final TextBox location = new TextBox();
    final Button button = new Button("Search");

    DOM.addEventPreview(new EventPreview() {
        public boolean onEventPreview(Event event) {
            if ( ! isEventTarget(event, button) &&
                 ! isEventTarget(event, location)) {
                switch (DOM.eventGetType(event)) {
                    case Event.ONMOUSEDOWN:
                    case Event.ONMOUSEUP:
                    case Event.ONMOUSEMOVE:
                    DOM.eventPreventDefault(event);
                }
            }
        return true;
        }
        private boolean isEventTarget(Event event,
                                      UIObject uio) {
            return DOM.eventGetTarget(event) == uio.getElement();
        }
    });

    final FocusPanel fp = new FocusPanel();
    final HorizontalPanel hp = new HorizontalPanel();
    final HTML html = new HTML();

    location.setVisibleLength(20);
    location.setMaxLength(100);

    location.addKeyboardListener(new KeyboardListenerAdapter() {
        public void onKeyPress(Widget sender, char keyCode, int mods) {
            if (keyCode == KEY_ENTER)
                button.click();
        }
    });
    DeferredCommand.add(new Command() {
        public void execute() {
            location.setFocus(true);
        }
    });

    hp.add(new HTML("Search for trips to&nbsp;"));
    hp.add(location);

    VerticalPanel vp = new VerticalPanel();
    vp.add(hp);
    vp.add(button);

    fp.add(vp);
    fp.addMouseListener(new FollowsMouseDragger());
    RootPanel.get().add(vp);

    button.addClickListener(new ClickListener() {
        public void onClick(Widget sender) {
          html.setVisible(false);

          TripServiceAsync ws = (TripServiceAsync)
              GWT.create(TripService.class);

          ServiceDefTarget target = (ServiceDefTarget) ws;
          target.setServiceEntryPoint(GWT.getModuleBaseURL()
                                      + "/tripService");
          ws.getTrips(location.getText(), new AsyncCallback() {
              public void onSuccess(Object result) {
                 Trip[] trips = (Trip[])result;
                 ListDragPanel p = new ListDragPanel();

                 p.addStyleName("tripsPanel");
                 p.setWidth("800px");
                 p.setHeight("400px");

                 for (int i=0; i < trips.length; ++i) {
                     Trip trip = trips[i];
                     TripPanel tp = new TripPanel(trip);
                     tp.addStyleName("tripDetailsPanel");
                     tp.setWidth("800px");
                     p.add(tp);
                 }

                 ScrollPanel sp = new ScrollPanel(p);
                 sp.setWidth("600px");
                 sp.setHeight("300px");

                 VerticalPanel vp = new VerticalPanel();
                 Label heading =
                    new Label("Trips to " + location.getText());
                 vp.add(heading);
                 vp.add(sp);
                 vp.addStyleName("tripPanel");

                 RootPanel.get().add(vp);

                 p.setVisible(false);
                 applyEffect("Grow", p.getElement());
              }
              public void onFailure(Throwable t) {
                  t.printStackTrace();
                showAlert("RPC call failed: " + t.getMessage()
                           + " stack trace: " + t.getStackTrace());
              }
          });
        }
      });
  }
  private native void showAlert(String message) /*-{
      $wnd.alert(message);
  }-*/;
  private native void applyEffect(String effect,
                                  Element element) /*-{
     $wnd.Effect[effect](element);
  }-*/;
}
