package com.acme.server;

import javax.xml.parsers.DocumentBuilderFactory;
import com.acme.client.TripService;
import com.acme.client.Trip;
import com.google.gwt.user.server.rpc.RemoteServiceServlet;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.GetMethod;
import java.io.InputStream;

public class TripServiceImpl extends RemoteServiceServlet
                          implements TripService {
    private static final String APPID = "gaspe_geary";
    private static final long serialVersionUID = 1L;

    public Trip[] getTrips(String location) {
        return getTripsFromDocument(
                    getTripsDocumentFromWebService(location, APPID));
    }
    private Trip[] getTripsFromDocument(Document document) {
        NodeList results = (NodeList)document
            .getElementsByTagName("Result");
        Trip[] trips = new Trip[results.getLength()];

        for (int i=0; i < results.getLength(); ++i) {
            Element result = (Element)results.item(i);
            Element title  = (Element)result
                .getElementsByTagName("Title").item(0);
            Trip trip = new Trip(title
                .getFirstChild().getNodeValue());
            trip.setDuration(getDuration(result));
            trip.setSummary(getSummary(result));
            trip.setImage(getImage(result));

            Element url = (Element)result
                .getElementsByTagName("Url").item(1);
            trip.setUrl(url.getFirstChild().getNodeValue());

            Element yahooId = (Element)result
                .getElementsByTagName("YahooID").item(0);
            trip.setYahooId(yahooId.getFirstChild().getNodeValue());
            trips[i] = trip;
        }
        return trips;
    }
    private String getImage(Element element) {
        Element image = (Element)element
            .getElementsByTagName("Image").item(0);
        Element url = (Element)image
            .getElementsByTagName("Url").item(0);
        return url.getFirstChild().getNodeValue();
    }
    private String getSummary(Element element) {
        Element summary = (Element)element
            .getElementsByTagName("Summary").item(0);
        return summary.getFirstChild().getNodeValue();
    }
    private int getDuration(Element element) {
        Element duration = (Element)element
            .getElementsByTagName("Duration").item(0);
        String s = duration.getFirstChild().getNodeValue();
        return Integer.parseInt(s);
    }
    private Document getTripsDocumentFromWebService(String location,
                                                    String appid) {
        Document document = null;
        String url = "http://api.travel.yahoo.com/TripService/V1" +
                     "/tripSearch?appid=" + appid + "&query=" + location;
        HttpClient client = new HttpClient();
        GetMethod get = new GetMethod(url);
        try {
            int result = client.executeMethod(get);
            if (result == 200) {
                InputStream in = get.getResponseBodyAsStream();
                document = DocumentBuilderFactory.newInstance()
                .newDocumentBuilder().parse(in);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return document;
    }
    private Document getTripsDocumentFromFile() {
        Document document = null;
        try {
            document = DocumentBuilderFactory.newInstance()
            .newDocumentBuilder()
                .parse(getServletContext().getResourceAsStream("/tripSearch.xml"));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return document;
    }
}